--!parent
--!encoding=utf-8 äöü

--****************************************************************************
-- @file LSW CORE LIB: OpenGL
-- @author Valentin Schmidt
-- @version	 0.1
-- @requires xtra("OpenGL")
--****************************************************************************

property ancestor

property $

property _keyHandler
property _keyTarget

property _mouseHandler
property _mouseTarget

-- constants
property GL

----------------------------------------
-- @constructor
----------------------------------------
on new (me, props)
	
	-- win only
	if $.OS="mac" then return FALSE

	-- libs
	$.import("detacher")

	setCurrentDir(me.libdir&"win")
	$.loadXtras(me.libdir, ["OpenGL.x32"], [])
	me.ancestor = xtra("OpenGL").new()

	-- load OpenGL constants into namespace GL
	me.loadGLConstants()

	return me
end

----------------------------------------
--
----------------------------------------
on forget (me)
	--_player.alert("FORGET")
	if ilk(me.ancestor)=#instance then
		--me.ancestor.forget()
		me.ancestor = VOID
	end if
end

----------------------------------------
-- gl_win_set_key_callback object me, eventCallback, *eventCallbackTarget
----------------------------------------
on gl_win_set_key_callback (me, cbHandler, cbTarget)
	me._keyHandler = cbHandler
	if voidP(cbTarget) then 
		me._keyTarget = _movie
	else
		me._keyTarget = cbTarget
	end if		
	me.ancestor.gl_win_set_key_callback(#_keyCallback, me)
end

----------------------------------------
-- gl_win_set_mouse_callback object me, eventCallback, *eventCallbackTarget
----------------------------------------
on gl_win_set_mouse_callback (me, cbHandler, cbTarget)
	me._mouseHandler = cbHandler
	if voidP(cbTarget) then 
		me._mouseTarget = _movie
	else
		me._mouseTarget = cbTarget
	end if		
	me.ancestor.gl_win_set_mouse_callback(#_mouseCallback, me)
end

----------------------------------------
-- utility: creates blank texture
----------------------------------------
on createTexture (me, w, h)
	res = []
	glGenTextures(1, res)
	texID = res[1]
	glBindTexture(GL.TEXTURE_2D, texID)
	glTexImage2D(GL.TEXTURE_2D, 0, GL.RGB, w, h, 0, GL.BGR, GL.UNSIGNED_BYTE)
	glTexParameterf(GL.TEXTURE_2D, GL.TEXTURE_MIN_FILTER, GL.LINEAR)
	glTexParameterf(GL.TEXTURE_2D, GL.TEXTURE_MAG_FILTER, GL.LINEAR)
	glTexParameteri(GL.TEXTURE_2D, GL.TEXTURE_WRAP_S, GL.REPEAT)
	glTexParameteri(GL.TEXTURE_2D, GL.TEXTURE_WRAP_T, GL.REPEAT)
	return texID
end

----------------------------------------
-- utility: loads texture from bitmap file using ImgXtra
----------------------------------------
on loadTexture (me, fn, texID)
	$.import("img")

	img = $.img.ix_loadImage(["filename":fn])
	data = $.img.ix_imageToRaw([#image:img])

	if voidP(texID) then
		res = []
		glGenTextures(1, res)
		texID = res[1]
	end if
	
	-- "bind" the newly created texture: all future texture functions will modify this texture
	glBindTexture(GL.TEXTURE_2D, texID)

	if img.useAlpha then
		-- 32-bit
		glTexImage2D(GL.TEXTURE_2D, 0, GL.RGBA, img.width, img.height, 0, GL.BGRA, GL.UNSIGNED_BYTE, data)
	else
		-- 24-bit
		glTexImage2D(GL.TEXTURE_2D, 0, GL.RGB, img.width, img.height, 0, GL.BGR, GL.UNSIGNED_BYTE, data)
	end if

	glTexParameterf(GL.TEXTURE_2D, GL.TEXTURE_MIN_FILTER, GL.LINEAR)
	glTexParameterf(GL.TEXTURE_2D, GL.TEXTURE_MAG_FILTER, GL.LINEAR)
	glTexParameteri(GL.TEXTURE_2D, GL.TEXTURE_WRAP_S, GL.REPEAT)
	glTexParameteri(GL.TEXTURE_2D, GL.TEXTURE_WRAP_T, GL.REPEAT)

	return texID
end

----------------------------------------
-- @callback
----------------------------------------
on _keyCallback (me, p1,p2,p3,p4)
	$.detacher.detach(me._keyHandler, me._keyTarget, [p1,p2,p3,p4])
end

----------------------------------------
-- @callback
----------------------------------------
on _mouseCallback (me, p1,p2,p3,p4,p5)
	$.detacher.detach(me._mouseHandler, me._mouseTarget, [p1,p2,p3,p4,p5])
end

----------------------------------------
-- Initialize gOpenGL Constants
----------------------------------------
on loadGLConstants (me)
	GL = [:]
	
	GL[#ARRAY_BUFFER] = 34962
	GL[#ELEMENT_ARRAY_BUFFER] = 34963
	GL[#FLOAT] = 5126
	GL[#FALSE] = 0
	GL[#TRUE] = 1
	GL[#FRAGMENT_SHADER] = 35632
	GL[#VERTEX_SHADER] = 35633
	GL[#STATIC_DRAW] = 35044

	GL[#COMPILE_STATUS] = 35713

	GL[#RGB] = 6407
	GL[#RGBA] = 6408

	GL[#BGR] = 32992
	--GL[#BGR_EXT] = GL[#BGR]
	GL[#BGRA] = 32993

	GL[#BLEND] = 3042
	GL[#CCW] = 2305
	GL[#COLOR_BUFFER_BIT] = 16384
	GL[#CULL_FACE] = 2884
	GL[#DEPTH_BUFFER_BIT] = 256
	GL[#DEPTH_TEST] = 2929
	GL[#LINEAR] = 9729
	GL[#MODELVIEW] = 5888	
	GL[#NEAREST] = 9728
	GL[#ONE_MINUS_SRC_ALPHA] = 771
	GL[#PROJECTION] = 5889
	GL[#QUADS] = 7
	GL[#REPEAT] = 10497
	GL[#SMOOTH] = 7425
	GL[#SRC_ALPHA] = 770
	GL[#TEXTURE_2D] = 3553
	GL[#TEXTURE_MAG_FILTER] = 10240
	GL[#TEXTURE_MIN_FILTER] = 10241
	GL[#TEXTURE_WRAP_S] = 10242
	GL[#TEXTURE_WRAP_T] = 10243
	GL[#TRIANGLES] = 4
	GL[#UNPACK_ALIGNMENT] = 3317
	GL[#UNPACK_ROW_LENGTH] = 3314
	GL[#UNPACK_SKIP_PIXELS] = 3316
	GL[#UNPACK_SKIP_ROWS] = 3315
	GL[#UNSIGNED_BYTE] = 5121
	GL[#UNSIGNED_INT] = 5125
	
	GL[#TEXTURE_WIDTH] = 4096
	GL[#TEXTURE_HEIGHT] = 4097

--#define GL_BYTE                           0x1400
--#define GL_UNSIGNED_BYTE                  0x1401
--/* #define GL_SHORT                          0x1402 */
--/* #define GL_UNSIGNED_SHORT                 0x1403 */
--#define GL_INT                            0x1404
--#define GL_UNSIGNED_INT                   0x1405
--#define GL_FLOAT                          0x1406
end
